
#include <ql/quantlib.hpp>
#include "models.h"

#ifdef BOOST_MSVC
/* Uncomment the following lines to unmask floating-point
   exceptions. Warning: unpredictable results can arise...

   See http://www.wilmott.com/messageview.cfm?catid=10&threadid=9481
   Is there anyone with a definitive word about this?
*/
// #include <float.h>
// namespace { unsigned int u = _controlfp(_EM_INEXACT, _MCW_EM); }
#endif

#include <boost/timer.hpp>
#include <iostream>
#include <iomanip>

using namespace QuantLib;

#if defined(QL_ENABLE_SESSIONS)
namespace QuantLib {

    Integer sessionId() { return 0; }

}
#endif

int probar()
{
    try {
        using namespace std;

        Calendar calendar = TARGET();
        Date todaysDate(6, February, 2011);
        Settings::instance().evaluationDate() = todaysDate;
        DayCounter dayCounter = Actual365Fixed();

        // Option parameter
        Option::Type type(Option::Put);
        Real underlying = 50;
        Real strike = 47;
        Spread dividendYield = 0.00;
        Rate riskFreeRate = 0.05;
        Volatility volatility = 0.20;
        Date maturity(10, December, 2011);

        // European exercise type handler
        boost::shared_ptr<Exercise> europeanExercise(
                new EuropeanExercise(
                maturity));

        // Quote (=underlying price) handler
        Handle<Quote> underlyingH(
                boost::shared_ptr<Quote>(
                new SimpleQuote(underlying)));

        // Yield term structure handler
        Handle<YieldTermStructure> flatTermStructure(
                boost::shared_ptr<YieldTermStructure>(
                new FlatForward(
                todaysDate,
                riskFreeRate,
                dayCounter)));

        // Dividend handler
        Handle<YieldTermStructure> flatDividendTermStructure(
                boost::shared_ptr<YieldTermStructure>(
                new FlatForward(
                todaysDate,
                dividendYield,
                dayCounter)));

        // Volatility handler
        Handle<BlackVolTermStructure> flatVolTermStructure(
                boost::shared_ptr<BlackVolTermStructure>(
                new BlackConstantVol(
                todaysDate,
                calendar,
                volatility,
                dayCounter)));

        // Payoff handler
        boost::shared_ptr<StrikedTypePayoff> payoff(
                new PlainVanillaPayoff(
                type,
                strike));

        // Black Scholes
        boost::shared_ptr<BlackScholesMertonProcess> bsmProcess(
                new BlackScholesMertonProcess(
                underlyingH,
                flatDividendTermStructure,
                flatTermStructure,
                flatVolTermStructure));

        // Option characteristics
        VanillaOption europeanOption(payoff, europeanExercise);

        // Pricing Engine : in this case BS for European options
        europeanOption.setPricingEngine(boost::shared_ptr<PricingEngine>(
                new AnalyticEuropeanEngine(
                bsmProcess)));

        /**************
        *  OUTPUTTING *
        ***************/

        // 1) Option parameters
        cout << "Option type =\t\t" << type << endl;
        cout << "Maturity =\t\t" << maturity << endl;
        cout << "Underlying price =\t" << underlying << endl;
        cout << "Strike =\t\t" << strike << endl;
        cout << "Risk-free int. rate =\t" << setprecision(2) << io::rate(riskFreeRate) << endl;
        cout << "Dividend yield =\t" << io::rate(dividendYield) << endl;
        cout << "Volatility =\t\t" << setprecision(2) << io::volatility(volatility) << endl;
        cout << endl;
        cout << endl;

        // 2) Calculation results
        cout << "Option price :\t" << setprecision(5) << europeanOption.NPV() << endl;
        cout << "Delta :\t\t" << setprecision(5) << europeanOption.delta() << endl;
        cout << "Elasticity :\t" << setprecision(5) << europeanOption.elasticity() << endl;
        cout << "Gamma :\t\t" << setprecision(5) << europeanOption.gamma() << endl;
        cout << "Vega :\t\t" << setprecision(5) << europeanOption.vega()/100 << endl;
        cout << "Theta :\t\t" << setprecision(5) << europeanOption.thetaPerDay() << endl;
        cout << "Rho :\t\t" << setprecision(5) << europeanOption.rho()/100 << endl;
        cout << "Implied vol. :\t" << setprecision(2) <<  io::rate(europeanOption.impliedVolatility(
                europeanOption.NPV(),
                bsmProcess,
                1.0e-4,	//
                100,	// please, see the QuantLib documentation
                1.0e-7, //
                1.0e3)) << endl;
        cout << endl;

        return 0;

    }
    catch (std::exception& e)
    {
            std::cerr << e.what() << std::endl;
            return 1;
    }
    catch (...)
    {
            std::cerr << "unknown error" << std::endl;
            return 1;
    }
}
